
controls = {}

local old_kbdisdown = love.keyboard.isDown
love.keyboard.isDown = function(key,...)
   return old_kbdisdown(key,...) or love.keyboard.isScancodeDown(key,...)
end


function controls:setScreenSize(w,h,dw,dh, font)
   self.width=w
   self.height = h
   self.dwidth = dw
   self.dheight = dh
   self.scaley = h/dh
   self.scalex = w/dw
   self.pxs = love.window.getPixelScale()
   self.font = font
   self.buttons = {
       left     = {left = 05*self.pxs,             top = self.dheight - 60*self.pxs, right = 25*self.pxs,             bottom = self.dheight-40*self.pxs,             sym="<" },
       right    = {left = 50*self.pxs,             top = self.dheight - 60*self.pxs, right = 70*self.pxs,             bottom = self.dheight-40*self.pxs,             sym=">" },
       up       = {left = 28*self.pxs,             top = self.dheight - 85*self.pxs, right = 48*self.pxs,             bottom = self.dheight-65*self.pxs,             sym="^" },
       down     = {left = 28*self.pxs,             top = self.dheight - 35*self.pxs, right = 48*self.pxs,             bottom = self.dheight-15*self.pxs,             sym="v" },
       action1  = {left = self.dwidth-70*self.pxs, top = self.dheight - 35*self.pxs, right = self.dwidth-50*self.pxs, bottom = self.dheight-15*self.pxs , float = 1, sym="A"},
       action2  = {left = self.dwidth-30*self.pxs, top = self.dheight - 35*self.pxs, right = self.dwidth-10*self.pxs, bottom = self.dheight-15*self.pxs , float = 1, sym="B"},
       action3  = {left = self.dwidth-70*self.pxs, top = self.dheight - 65*self.pxs, right = self.dwidth-50*self.pxs, bottom = self.dheight-45*self.pxs , float = 1, sym="X"},
       action4  = {left = self.dwidth-30*self.pxs, top = self.dheight - 65*self.pxs, right = self.dwidth-10*self.pxs, bottom = self.dheight-45*self.pxs , float = 1, sym="Y"},
       action5  = {left = self.dwidth-30*self.pxs, top = 10*self.pxs,                right = self.dwidth-10*self.pxs, bottom = 30*self.pxs,               float = 1, sym="I"},
   }
end

function controls:set_game(gamedef)
   self.gamedef = gamedef
   local offset = 0
   local ww, wh = love.window.getMode()
   self.offset = (ww/gamedef.scaley - gamedef.default_window_width)/2
end

function controls:enableButtons(buttons)
   self.enabledButtons={}
   for k,v in ipairs(buttons) do
       self.enabledButtons[v]=true
   end
end

function controls.setMode(mode)
   controls.mode = mode
   if mode == "menu" then
      controls.enableButtons({'up','down','left','right','action1','action2','action3','action4', 'action5'})
   end  
end


local lastbutton = ""
controls.enabledButtons={}

function controls:clear()
   for k,v in pairs(self.buttons) do
       if self.enabledButtons[k] then
          self['moving_'..k]=false
       end
   end
end

function controls:update(dt, keys)
    if not keys then print("no keys no gains") return end
    if self.hidden then return end
    local s = ""
    for k,v in pairs(self.gamedef.keys) do
        if v then
           s = s..k..", "
        end
    end
    lastbutton = s
    self.pressed_buttons_list = s    

    if love.keyboard.isDown('escape') or lastbutton:find('escape') then self.escape = true else self.escape = false end

      self.not_pressed = {}
      if not (self.not_pressed.kbd_left    or self.not_pressed.left_str)    then self.moving_left = false self.x1 = true end
      if not (self.not_pressed.kbd_right   or self.not_pressed.right_str)   then self.moving_right = false self.x2 = true end
      if not (self.not_pressed.kbd_down    or self.not_pressed.down_str)    then self.moving_down = false self.x2 = true end
      if not (self.not_pressed.kbd_up      or self.not_pressed.up_str)      then self.moving_up = false self.x2 = true end
      if not (self.not_pressed.kbd_action1 or self.not_pressed.action1_str) then self.action1 = false self.moving_action1 = false end
      if not (self.not_pressed.kbd_action2 or self.not_pressed.action2_str) then self.action2 = false self.moving_action2 = false end
      if not (self.not_pressed.kbd_action3 or self.not_pressed.action3_str) then self.action3 = false self.moving_action3 = false end
      if not (self.not_pressed.kbd_action4 or self.not_pressed.action4_str) then self.action4 = false self.moving_action4 = false end
      if not (self.not_pressed.kbd_action5 or self.not_pressed.action5_str) then self.action5 = false self.moving_action5 = false end

      local collision
      self.moving_down = nil
      self.moving_left = nil
      self.moving_right = nil
      self.moving_action1 = nil
      self.moving_action2 = nil
      self.moving_action3 = nil
      self.moving_action4 = nil
      self.moving_action5 = nil
      self.moving_up = nil

      if keys.up and love.keyboard.isDown(keys.up) then self.moving_up   = true else self.not_pressed.kbd_up  =true end
      if keys.down and love.keyboard.isDown(keys.down) then self.moving_down = true else self.not_pressed.kbd_down=true end
      if keys.left and love.keyboard.isDown(keys.left) then self.moving_left=true else self.not_pressed.kbd_left = true end
      if keys.right and love.keyboard.isDown(keys.right) then self.moving_right=true else self.not_pressed.kbd_right = true end
      if keys.action1 and love.keyboard.isDown(keys.action1) then self.moving_action1=true else self.not_pressed.kbd_action1 = true end
      if keys.action2 and love.keyboard.isDown(keys.action2) then self.moving_action2=true else self.not_pressed.kbd_action2 = true end
      if keys.action3 and love.keyboard.isDown(keys.action3) then self.moving_action3=true else self.not_pressed.kbd_action3 = true end
      if keys.action4 and love.keyboard.isDown(keys.action4) then self.moving_action4=true else self.not_pressed.kbd_action4 = true end
      if keys.action5 and love.keyboard.isDown(keys.action5) then self.moving_action5=true else self.not_pressed.kbd_action5 = true end



      if self.moving_action1 then self.action1 = true else self.action1 = false end
      if self.moving_action2 then self.action2 = true else self.action2 = false end
      if self.moving_action3 then self.action3 = true else self.action3 = false end
      if self.moving_action4 then self.action4 = true else self.action4 = false end
      if self.moving_action5 then self.action5 = true else self.action5 = false end

      if self.action1 and not self.action1_pressed then self.can_action1=true else self.can_action1=false  end
      if self.action2 and not self.action2_pressed then self.can_action2=true else self.can_action2=false  end
      if self.action3 and not self.action3_pressed then self.can_action3=true else self.can_action3=false  end
      if self.action4 and not self.action4_pressed then self.can_action4=true else self.can_action4=false  end
      if self.action5 and not self.action5_pressed then self.can_action5=true else self.can_action5=false  end

      if self.action1 then self.action1_pressed = true else self.action1_pressed = false end
      if self.action2 then self.action2_pressed = true else self.action2_pressed = false end
      if self.action3 then self.action3_pressed = true else self.action3_pressed = false end
      if self.action4 then self.action4_pressed = true else self.action4_pressed = false end
      if self.action5 then self.action5_pressed = true else self.action5_pressed = false end


end


function love.gamepadpressed(joystick, button)
    lastbutton = button
end

function controls:draw(tx,ty)
end

return controls
