
local players = {}

local anim8       = require 'lib.anim8'
local controls    = require "controls"



function players:set_game(gamedef)
   players.gamedef = gamedef   
end

function players:add(name, params)
  local params = params or {}
  if not name and not params.name then return nil, "У игрока *должно* быть имя!" end
  local name = name or params.name
  local same_name
  for k,v in pairs (players) do
      if type(v)=="table" and v.is_player then
         if v.name == name then
            same_name = true
            break
         end
	  end
  end
  if samename then return nil, "Игрок с таким именем уже существует!" end

  local player_image = love.graphics.newImage(params.image or 'gfx/Gum Bot sprites.png')
  local player_grid = anim8.newGrid(32, 32, player_image:getWidth(), player_image:getHeight())

  local player = {x         = params.x or 340,
                  y         = params.y or 1000,
                  w         = params.w or 16,
                  h         = params.h or 16,
				  max_hp    = params.max_hp or 10, 
				  hp        = params.max_hp or 10,
				  hurt      = params.hurt or 0,
                  velx      = params.velocity_y or 0,
                  vely      = params.velocity_x or 0,
                  mass      = params.mass or 100,
                  mvx       = params.max_velocity_x or 200,
                  hpw       = params.horse_power or 100,
                  dir       = params.dir or 1,
                  grnd      = params.on_grond or false,
                  gravity   = params.gravity or 9.8*60,
                  is_player = true,
                  name      = name,
				  powerups  = {},
                  image     = player_image,
                  gfx       = { grid             = player_grid,
                                flip             = 1,
                                animation        = { walk  = anim8.newAnimation(player_grid('2-7',  3), 0.1),
                                                     idle  = anim8.newAnimation(player_grid(    3,  2), 600),
                                                     face  = anim8.newAnimation(player_grid(    2,  2), 600),
                                                     jump  = anim8.newAnimation(player_grid(    6,  3), 600),
                                                     climb = anim8.newAnimation(player_grid('6-8',  4), 0.1),
                                                     turn  = anim8.newAnimation(player_grid('4-6',  2), 0.1, function(self) self:pauseAtEnd() end),
                                                   },
                               current_animation = "walk",
                              },
                  jumped    = false,
				  messages  = message,
                }

   player.coins = 1000000
   player.controls = controls
   player.controls:set_game(self.gamedef)
   player.controls_drawn = false
   if not params.keys then params.keys = {} end
   local major, minor, revision, codename = love.getVersion( )
   local space_char = "space"
   if minor<10 then space_char=" " end
   player.keys = {up      = params.keys.up      or "w",
                  down    = params.keys.down    or "s",
				  left    = params.keys.left    or "a",
				  right   = params.keys.right   or "d",
				  action1 = params.keys.action1 or "return",
				  action2 = params.keys.action2 or "backspace",
				  action3 = params.keys.action3 or "q",
				  action4 = params.keys.action4 or "e",
				  action5 = params.keys.action5 or space_char
			     }

   w  = players.gamedef.width
   h  = players.gamedef.height
   dw = players.gamedef.default_window_width
   dh = players.gamedef.default_window_height

   player.controls:setScreenSize(w,h, dw, dh, players.gamedef.font)
   player.controls:enableButtons({'up','down','left','right','action1','action2','action3','action4', "action5"})

   player.image:setFilter("nearest","nearest",0,0)

   function player:filter(other)
      local controls = self.controls
	  local items  = players.gamedef.items
	  if other.type == "instadeath" then
	     self.hp = 0

	  end
	  if other.type == "spikes" then
	     self.hp = 0

	  end

	  if other.is_player then
	     if other.mass*other.hpw>self.mass*self.hpw then
		 end
	     return "slide"
	  end

     if other.is_sensor then
	    return false
	 end

  
      if other.type == "climbable" then
         if not controls.moving_action1 and (controls.moving_up or self.grabbed) then
            self.grabbed = true
            self.can_climb = true
            self.grnd = true
            self.jumped = false
            self.alignx = other.x + (other.width-self.w)/2
            return "cross"
        end

        if not controls.moving_action1 and (controls.moving_down or self.grabbed) then
           self.can_climb = true
           self.grabbed = true
           self.jumped = false
           self.alignx = other.x + (other.width-self.w)/2
           return "cross"
        end

        if other.properties.solid == "top" and self.y + self.h <= other.y and not (controls.moving_down or self.grabbed) then
           self.alignx = nil
           self.grnd = true
           self.can_climb = false
           return "slide"
        end
        return false
     end

      if other.properties then
	     if other.properties.solid == "top" then
            if controls.moving_action1 and self.grnd and controls.moving_down then
               return false
			end
		    if self.y + self.h > other.y then
               return false
            end
		 end
      end
	  if other.projectile then
		 return false
	  end

      self.grabbed = false

	  return "slide"
   end

   self[name]=player
   players.gamedef.player_list = players.gamedef.player_list or {}
   players.gamedef.player_list[player.name] = player
   self.gamedef.ps.world:add(player, player.x, player.y, player.w, player.h)

   player.sensors = {}
   
   function player.sensors.filter(sensor,other)
	  if other.destructible then
	     return "slide"
	  else
	     return false
	  end
   end

   player.sensors.b = {w = player.w, h = 2, xoff = 0, yoff = 6, is_sensor = true}
   self.gamedef.ps.world:add(player.sensors.b, player.sensors.b.xoff+player.x, player.sensors.b.yoff+player.y, player.sensors.b.w, player.sensors.b.h)
   
   return player

end

function players:update(dt) 
   for pll, player in pairs(self) do
       if type(player) == "table" and not player.is_dead and player.is_player then
	      
	      local do_not_update 
	      
		  if player.hp <= 0 then
		     if not player.death_boom_timer then 
		        player.death_boom_timer = 5 
		     end
		     player.death_boom_timer = player.death_boom_timer - dt
		     if player.death_boom_timer > 0 then
		        do_not_update = true
		        player.vely=0
		        player.velx=0
		        if not player.old_gravity then
  		           player.old_gravity = player.gravity + 0
		           player.gravity = 0		        
		        end
		     else				 
		         player.death_boom_timer = nil
		         if player.old_gravity then
		            player.gravity = player.old_gravity
		            player.old_gravity = nil
		         end 		         
				 player.x = 32
				 player.y = 32
				 player.hp = player.max_hp
				 self.gamedef.ps.world:update(player, player.x, player.y,16,16)			 
			 end
		  end 
		  player.hurt = player.hurt-dt
		  player.flash = not player.flash
		  if player.hurt<=0 then player.hurt = 0 end	  
		 for k, v in pairs(player.powerups) do
			 player.powerups[k] = player.powerups[k] - dt
			 if player.powerups[k] <=0 then			    
				player.powerups[k] = nil				
			 end
		 end


         if do_not_update or player.hurt>0 then
            player.controls:clear()
		 else
		    player.controls:update(dt,player.keys)
		 end

          local gfx = player.gfx
          local x, y   = player.x, player.y
          local vx, vy = player.velx, player.vely
          local grav = player.gravity + 0
          local controls = player.controls
		  local gamedef = self.gamedef

          if gfx.current_animation ~= "face" then
             gfx.current_animation  = "idle"
          end



         
			 if controls.moving_left or controls.moving_right then
				local dir = player.dir
				if controls.moving_left then
				   player.dir = -1
				else
				   player.dir = 1
				end

				if dir~=player.dir then
				   vx = vx - (player.mass*dt)*0.7*player.mvx
				   if vx<0 then
					  vx = 0
				   end
				   if not player.grabbed then
					  gfx.current_animation  = "face"
					  player.dont_update_anim = "face"
					  player.dont_update_anim = nil
				   end
				else
				   vx = vx + player.mvx*dt
				   if not player.grabbed then
					  gfx.current_animation = "walk"
				   end
				end
			 else
				vx = vx - (player.mass/100)*0.7*player.mvx
				if vx<0 then
				   vx = 0
				end

				gfx.current_animation = "idle"
			 end

			 if player.grabbed then
				gfx.current_animation = "climb"
			 end

			 player.recharge = (player.recharge or 0 )+ dt
			 local Hoffset = 0 if player.dir<0 then Hoffset = 2 else Hoffset = 8  end

			 if not controls.moving_action2 then
				if item and item.melee then
				   player.shot_timer = 0
				   player.shot = false
				end
				player.need_to_release_fire = false
			 end

			 if player.jumped then
				gfx.current_animation = "jump"
			 end

			 if player.can_climb then
				grav = 0
			 end

             
             local vo = vy+0			 
			 vy = vy + (grav+player.mass)*dt
			 local dy = (vo + vy) * dt * 0.5
			 if player.can_climb then
			    dy = 0
			 end
			 player.dy_____ = dy
			 
			 if vx > player.mvx then
   				vx = player.mvx
			 end

			 if vx < 0 then
				vx = 0
			 end

			 if player.grabbed and controls.moving_action1 then
				player.grabbed = false
			 end

			 if controls.moving_up and not player.can_climb and not player.jumped and not controls.moving_left and not controls.moving_right and not controls.moving_action1 and not controls.moving_action2 and not controls.moving_down then
				player.turn_timer = (player.turn_timer or 0.3) - dt
				gfx.current_animation = "turn"
				gfx.animation.turn:resume()
			 else
				gfx.animation.turn:pauseAtStart()
				player.turn_timer = nil
			 end

			 if controls.moving_up then
				if player.can_climb then
				   dy = -100*dt
				   player.grnd = false
				end
			 end

			 if controls.moving_down then
				if player.can_climb then
				   dy = 100*dt
				   player.grnd = false
				end
			 end

			 x = x + player.dir*vx*dt
			 if player.hurt>0 then
			    x = x - 32*dt*player.dir*2
			 end

			 if player.alignx then
				x = player.alignx
				player.alignx = nil
			 end

			 y = y + dy
			 player.velx, player.vely = vx, vy

			 local cols = {}
			 local len = 0
			 player.old_x, player.old_y = player.x+0, player.y+0
			 player.glued = nil
			 player.x, player.y, cols, len = gamedef.ps.world:move(player, x, y, player.filter)   
			 for k,v in pairs(player.sensors) do
			     if type(v)=="table" then
			        local cc, ll					
			        player.sensors[k].x, player.sensors[k].y, cc, ll = gamedef.ps.world:check(v, player.x+v.xoff, player.y+v.yoff+10, player.sensors.filter)
					if ll>0 then
					   for i,col in ipairs(cc) do
					       if cc[i].normal.y == -1 then
						      player.on_destructible = true
							  break
						   end
					   end
					else
					   player.on_destructible = false
					end 

			        gamedef.ps.world:update(v, player.x+v.xoff, player.y+v.yoff+10)

				 end
			 end
			 
			 if len == 0 then
				player.jumped=true
				player.can_climb = false
				player.grabbed = false
				player.grnd = false				
			 end

			 if player.glued then
				player.x = player.glued				
			 end

			 for i,v in ipairs (cols) do			     
					if cols[i].normal.y == -1 or player.on_destructible then 
					   player.vely = 0
					   player.grnd = true
					   if player.sfx and player.sfx.ground and player.jumped then

						  player.sfx.ground:play()
					   end
					   player.jumped = false
					elseif cols[i].normal.y == 1 then
					   player.vely = player.vely/4
					   local other = cols[i].other
					end
					if cols[i].type~="cross" and cols[i].normal.x ~= 0 then
					   player.velx = 0
					end
			 end

			 if controls.moving_action1 and player.grnd and not player.jumped then
				player.jumped = true
				if not controls.moving_down then
				   player.vely = -player.hpw*1.2*1/80*player.mvx + player.mass*0.7
				   player.vely = player.vely + player.vely*player.velx*0.001
				end
				player.grnd = false
			 end

			 player.fore = false
			 if controls.moving_action1 and controls.moving_down then
				player.fore = true
			 end


			 if player.dont_update_anim then
				gfx.current_animation = player.dont_update_anim
			 end

			 if (gfx.current_animation=="climb" and (controls.moving_up or controls.moving_down))
			 or gfx.current_animation~="climb"
			 then
				gfx.animation[gfx.current_animation]:update(dt)
			 end
      end
   end
end


function players:draw_player(pll, tx, ty)
    local player = players[pll]
			if player.death_boom_timer then 			   
               love.graphics.push()
               love.graphics.translate(player.x,player.y)
               love.graphics.print(math.ceil(player.death_boom_timer), 4,4)
               love.graphics.pop()
			   return 
			end
          love.graphics.push()
          love.graphics.translate(player.x,player.y)
	      local game = players.gamedef
	      local gfx = player.gfx

          local Hoffset = 0 if player.dir<0 then Hoffset = 32 end
		  if player.hurt > 0 then
		     if player.flash then
		        love.graphics.setColor(0,0,0,0)
             end
		  end
          gfx.animation[gfx.current_animation]:draw(player.image, Hoffset-8, -16, 0, player.dir, 1)
		  if player.hurt > 0 then
		     love.graphics.setColor(255,255,255)
		  end
	  love.graphics.pop()
	  
end

function players:draw(tx,ty)
   local offset = 0
   for pll, player in pairs(self) do
       if type(player) == "table" then
          if not player.death_boom_timer then
              players:draw_player(player.name, tx, ty)
          end		  
       end
   end
end

return players
